import IndicoWp.config as config
import IndicoWp.indico.event as event
import logging

import requests
import json
import urllib.parse as urlparse


# TODO: Make this better with making a response dict a current item and a dict query


class IndicoEventRequestController:
    """
    :ivar config: The config parser object for the project
    :ivar logger: The logger for 'IndicoRequest'
    :ivar base_url: The url of the indico website which is to be used for the program
    :ivar api_key: The api key to be used to identify the requests to the api of the site
    :ivar processor: The IndicoEventProcessor, that turns the dict structures containing the infos about the events
        into the Event objects
    """
    def __init__(self):
        self.config = config.Config.get_instance()
        self.logger = logging.getLogger('IndicoRequest')

        self.base_url = None
        self.api_key = None

        self.processor = IndicoEventProcessor()

    def set(self, observation):
        self.base_url = observation.url
        self.api_key = observation.key

    def set_url(self, base_url):
        self.base_url = base_url

    def set_key(self, key):
        self.api_key = key

    def get_category_events(self, category_id):
        """
        A list of Event objects for each event in the category given by its id.

        :param category_id: The id of the indico category for which to get the events
        :return: [event.Event]
        """
        if self.base_url is None:
            raise ValueError('no base url set for IndicoEventRequestController')

        response_dict = self.request_category(category_id)

        event_dict_list = response_dict['results']
        event_list = []
        for event_dict in event_dict_list:
            indico_event = self.processor.process(event_dict)
            event_list.append(indico_event)

        return event_list

    def request_category(self, category_id):
        """
        Sends a request for the given category id and returns the json decoded dict structure of the response

        :param category_id: The id of the indico category for which to get the events
        :return: dict
        """
        # Assembling the part of the url, that uses the category id
        category_string = '/categ/{}.json'.format(category_id)

        url_query = {
            'apikey': self.api_key
        }

        url = '{}/{}?{}'.format(
            self.base_url,
            category_string,
            urlparse.urlencode(url_query)
        )

        print(url)

        response = requests.get(url)
        response_dict = json.loads(response.text)
        return response_dict


class IndicoEventProcessor:
    """
    :ivar dict: The dict object containing the info about the event, that is being processed by the object at a given
        moment in time. Is being changed with each new event to process
    :ivar logger: The logger for IndicoProcessing
    :cvar _DEBUG_STRING_CHARACTER_COUNT: In case a item is missing in the processed dict a log entry is being made and
        to identify to which event the log message belonged, a string of the dict that is processed is being appended
        this variable specifies how many characters to use of that string
    """
    _DEBUG_STRING_CHARACTER_COUNT = 30

    def __init__(self):
        self.dict = None
        self.logger = logging.getLogger('IndicoProcessing')

    def process(self, event_dict):
        """
        Processes the given event dict into a event.Event object and returns that

        :param event_dict: The dict structure, that contains the event data
        :return: Event
        """
        self.dict = event_dict

        return self._event()

    def _event(self):
        """
        Creates the actual Event object from all the parameter objects

        :return: Event
        """
        return event.IndicoEvent(
            self._event_id(),
            self._event_description(),
            self._event_location(),
            self._event_time(),
            self._event_creator(),
            self._event_meta()
        )

    def _event_id(self):
        """
        The indico id for the event

        :return: the int id
        """
        return int(self._query_dict('id', 0))

    def _event_description(self):
        """
        Creates the EventDescription from the currently processed dict

        :return: EventDescription
        """
        description = self._query_dict('description', '')
        title = self._query_dict('title', '')
        event_type = self._query_dict('type', '')

        event_description = event.EventDescription(title, description, event_type)
        return event_description

    def _event_creator(self):
        """
        Created the EventCreator from the currently processed dict

        :return: EventCreator
        """
        creator_full_name = self._query_dict('creator/fullName', '')
        if not creator_full_name == '':
            creator_name_split = creator_full_name.split(',')
            creator_first_name = creator_name_split[1].replace(' ', '').lower()
            creator_last_name = creator_name_split[0].replace(' ', '').lower()
        else:
            creator_first_name = ''
            creator_last_name = ''
        creator_id = int(self._query_dict('creator/id', 0))
        creator_affiliation = self._query_dict('creator/affiliation', '')

        event_creator = event.EventCreator(creator_id, creator_first_name, creator_last_name, creator_affiliation)
        return event_creator

    def _event_time(self):
        """
        Created the EventTime object from the currently processed dict

        :return: EventTime
        """
        date = self._query_dict('startDate/date', '')
        time = self._query_dict('startDate/time', '')

        event_time = event.EventTime(time, date)
        return event_time

    def _creation_time(self):
        date = self._query_dict('creationDate/date', '')
        time = self._query_dict('creationDate/time', '')
        if '.' in time:
            time = time[:time.find('.')]

        event_time = event.EventTime(time, date)
        return event_time

    def _event_meta(self):
        creation_time = self._creation_time()
        url = self._query_dict('url', '')

        event_meta = event.EventMeta(url, creation_time)
        return event_meta

    def _event_location(self):
        """
        Created the EventLocation object from the dict.

        :return: EventLocation
        """
        location = self._query_dict('location', None)
        address = self._query_dict('address', None)

        event_location = event.EventLocation(location, address)
        return event_location

    def _query_dict(self, dict_query, default):
        """
        Uses a single string to perform a possibly multi layer get from the currently processed string. The keys for
        the dict structures have to be separated by '/' to indicate, that they are intended for the next layer.
        A default value has to be given in case the query fails.
        If the query fails a debug entry is added to the log file.

        :param dict_query: The string query for the dict
        :param default: The default value to be returned in case the query fails
        :return: The value in the deepest dict layer queried
        """
        try:
            keys = dict_query.split('/')
            current_dict = self.dict

            for key in keys:
                current_dict = current_dict[key]

            return current_dict
        except (KeyError, ValueError) as excpetion:
            self.logger.debug('There is no key "{}" in the event dict "{}..."'.format(
                dict_query,
                self._debug_string
            ))
            return default

    @property
    def _debug_string(self):
        """
        The first few character of a string format of the dict currently processed for recognition in debugging the
        log files.
        The exact amount of characters that is used can be set by the static class variable
        '_DEBUG_STRING_CHARACTER_COUNT'

        :return: The string of the dict
        """
        dict_string = str(self.dict)
        # Only taking the first few characters, the amount specified by the static field
        if len(dict_string) > self._DEBUG_STRING_CHARACTER_COUNT:
            dict_string = dict_string[self._DEBUG_STRING_CHARACTER_COUNT:]
        return dict_string



from IndicoWp.config import PATH

import pathlib
import MySQLdb

# THE FILE TEMPLATES

CONFIG_INI_TEMPLATE = (
    '[INDICO]\n'
    'observe = indico.ini\n\n'
    '[WORDPRESS]\n'
    'url = # Insert url to your wordpress site xmlrpc.php file\n'
    'username = # The username to the wordpress site\n'
    'password = # The password to the wordpress site\n\n'
    '[LOGGING]\n'
    'folder = logs\n\n'
    '[MYSQL]\n'
    'host = localhost\n'
    'database = # The name of the mysql database to use\n'
    'username = # The username to your mysql database\n'
    'password = # The password for the mysql user\n'
)

INDICO_INI_TEMPLATE = (
    '[SHORT NAME OF THE SITE]\n'
    'url = # The url to the indico site\n'
    'key = # The API Key to be used to request the information from the site\n'
    'categories = ["1", "3"] # A list of all the category ids to be observed\n\n'
    '# Multiple Indico sites can be observed by adding more sections like the example above, one for each site to\n'
    '# be monitored. The short name/ section title has to be unique for each one and also a new API key has to be \n'
    '# acquired on each site.'
)

ID_JSON_TEMPLATE = (
    '{"counter":0, "used":[], "unused":[]}'
)

# THE SQL TEMPLATES

PRE_SQL = (
    'SET foreign_key_checks=0; COMMIT;'
)

EVENT_TABLE_SQL = (
    'CREATE TABLE event ('
    'id INT(11) PRIMARY KEY NOT NULL,'
    '`starting` DATETIME,'
    'location VARCHAR(200),'
    'address VARCHAR(200),'
    'description TEXT,'
    'type VARCHAR(200),'
    'title VARCHAR(200),'
    'creator_id INT(11),'
    'FOREIGN KEY (creator_id) REFERENCES creator(id)'
    ');'
    'ALTER TABLE event ENGINE=INNODB;'
    'COMMIT;'
)

CREATOR_TABLE_SQL = (
    'CREATE TABLE creator ('
    'id INT(11),'
    'first_name VARCHAR(100),'
    'last_name VARCHAR(100),'
    'affiliation VARCHAR(300)'
    ');'
    'ALTER TABLE creator ENGINE=INNODB;'
    'COMMIT;'
)

EVENT_REFERENCE_TABLE_SQL = (
    'CREATE TABLE event_reference ('
    'internal_id INT(11),'
    'indico_id INT(11),'
    'wordpress_id INT(11)'
    ');'
    'ALTER TABLE event_reference ENGINE=INNODB;'
    'COMMIT;'
)

# SUPPORT CLASSES


class Db:

    _instance = None

    @staticmethod
    def get_cursor():
        conn = Db.get_instance()  # type: MySQLdb.Connection
        cursor = conn.cursor()
        return cursor

    @staticmethod
    def get_instance():
        if Db._instance is None:
            Db.new_instance()

        return Db._instance

    @staticmethod
    def new_instance():

        # Attempting to import the config of the project
        from IndicoWp.config import Config

        try:
            config = Config.get_instance()
            # Getting the values from the config
            host = config['MYSQL']['host']
            database = config['MYSQL']['database']
            username = config['MYSQL']['username']
            password = config['MYSQL']['password']
            connector = MySQLdb.connect(
                host=host,
                db=database,
                user=username,
                passwd=password
            )
            Db._instance = connector
        except:
            pass


# ACTUAL

class SetupController:

    def __init__(self):
        self.path = None
        self.prompt_path()

        self.config_controller = ProjectConfigController(str(self.path))
        self.file_controller = FilesSetupController(str(self.path))
        self.database_controller = DatabaseSetupController()

    def setup_database(self):
        self.database_controller.run()

    def setup_files(self):
        self.file_controller.run()
        self.config_controller.run()

    def prompt_path(self):
        while True:
            path_string = input('Enter project path:  ')
            path = pathlib.Path(path_string)
            if path.exists() and path.is_dir():
                break
            else:
                print('The path was not valid, please try again')

        self.path = path


class ConfigSetupController:

    def __init__(self, project_path):
        self.project_path = pathlib.Path(project_path)
        self.path = self.project_path / 'config.ini'  # type: pathlib.Path

    def run(self):
        if not self.exists():
            self.create()

    def create(self):
        with self.path.open(mode='w+') as file:
            file.write(CONFIG_INI_TEMPLATE)
            file.flush()

    def exists(self):
        return self.path.exists()


class IndicoSetupController:

    def __init__(self, project_path):
        self.project_path = pathlib.Path(project_path)
        self.path = self.project_path / 'indico.ini'

    def run(self):
        if not self.exists():
            self.create()

    def create(self):
        with self.path.open(mode='w+') as file:
            file.write(INDICO_INI_TEMPLATE)
            file.flush()

    def exists(self):
        return self.path.exists()


class IdJsonSetupController:

    def __init__(self, project_path):
        self.project_path = pathlib.Path(project_path)
        self.path = self.project_path / 'id.json'

    def run(self):
        if not self.exists():
            self.create()

    def create(self):
        with self.path.open(mode='w+') as file:
            file.write(ID_JSON_TEMPLATE)
            file.flush()

    def exists(self):
        return self.path.exists()


class ProjectConfigController:

    def __init__(self, project_path):
        self.project_path_string = project_path
        # Getting the file path for the config file of the project
        import IndicoWp.config as cfg
        self.config_path = pathlib.Path(cfg.__file__)
        self.content = ''

    def run(self):
        self.read()
        self.replace()
        self.write()

    def write(self):
        with self.config_path.open(mode='w') as file:
            file.write(self.content)
            file.flush()

    def read(self):
        with self.config_path.open(mode='r') as file:
            self.content = file.read()

    def replace(self):
        lines = self.content.split('\n')
        index = 0
        for line in lines:
            if 'PROJECT_PATH' in line:
                break
            index += 1
        lines[index] = 'PROJECT_PATH = "{}"'.format(self.project_path_string)
        self.content = '\n'.join(lines)


class FilesSetupController:

    def __init__(self, project_path):
        self.config_setup_controller = ConfigSetupController(project_path)
        self.indico_setup_controller = IndicoSetupController(project_path)
        self.id_setup_controller = IdJsonSetupController(project_path)

    def run(self):
        self.indico_setup_controller.run()
        self.config_setup_controller.run()
        self.id_setup_controller.run()


class DatabaseSetupController:

    def __init__(self):
        self.cursor = None

    def run(self):
        self.cursor = Db.get_cursor()

        self.execute(PRE_SQL)
        if not self.exists('creator'):
            self.execute(CREATOR_TABLE_SQL)
        if not self.exists('event'):
            self.execute(EVENT_TABLE_SQL)
        if not self.exists('event_reference'):
            self.execute(EVENT_REFERENCE_TABLE_SQL)

    def exists(self, database_name):
        try:
            sql = 'SELECT * FROM {db};'.format(database_name)
            self.execute(sql)
            return True
        except:
            return False

    def execute(self, sql):
        self.cursor.execute(sql)



from sqlalchemy import Column, Integer
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import Session

from IndicoWp.database import MySQLDatabaseAccess

from IndicoWp.config import Config, PATH

import logging
import pathlib
import json


Base = declarative_base()


class EventReference(Base):
    __tablename__ = 'event_reference'

    internal_id = Column(Integer, primary_key=True)
    indico_id = Column(Integer)
    wordpress_id = Column(Integer)


class ReferenceController:

    def __init__(self):
        self.session = MySQLDatabaseAccess.get_session()  # type: Session
        self.id_manager = IDManager()

    def close(self):
        self.id_manager.save()

    def insert_reference(self, internal_id, indico_id, wordpress_id):
        self.session.add(EventReference(
            internal_id=internal_id,
            indico_id=indico_id,
            wordpress_id=wordpress_id
        ))
        self.session.commit()

    def select_all(self):
        return self.session.query(EventReference)

    def event_from_indico_event(self, indico_event):
        # Replacing the indico id with the internal id generated by the id manager
        internal_id = self.id_manager.get()
        indico_event_copy = indico_event.copy()
        indico_event_copy.id = internal_id
        return indico_event_copy


class IDManager:

    def __init__(self):
        self.config = Config.get_instance()
        self.logger = logging.getLogger('IdManagement')

        self.path = pathlib.Path(PATH) / self.config['LOGGING']['ids']

        self.counter = None
        self.used = None
        self.load()

    def load(self):
        with self.path.open(mode='r') as file:
            content_dict = json.load(file)
            self.counter = content_dict['counter']
            self.used = content_dict['used']

    def save(self):
        with self.path.open(mode='w+') as file:
            content_dict = {
                'counter': self.counter,
                'used': self.used
            }
            json.dump(content_dict, file)

    def get(self):
        current_id = self.counter
        self.used.append(current_id)
        self.counter += 1
        return current_id




def indico_request():
    import requests
    import json
    import urllib.parse as urlparse
    from pprint import pprint

    api_key = '829e3826-39ad-4be3-b50f-1d25397e67bd'
    url_base = 'https://indico.desy.de/indico/export/categ/384.json'
    url_query = {
        'apikey': api_key
    }

    url = '{}?{}'.format(url_base, urlparse.urlencode(url_query))

    response = requests.get(url)
    response_dict = json.loads(response.text)
    pprint(response_dict)


def simple_text_view():
    from IndicoWp.indico.indico import IndicoEventRequestController
    from IndicoWp.views.text import SimpleEventTextView, SimpleEventsTableView

    requester = IndicoEventRequestController()
    event_list = requester.get_category_events(384)
    view = SimpleEventsTableView(event_list)
    print(view.get_string())


def create_database():
    from IndicoWp.database import MySQLDatabaseAccess
    MySQLDatabaseAccess.create_database()


def observerd():
    from IndicoWp.controller import Controller
    controller = Controller()
    try:
        controller.post_observed_events()
    finally:
        controller.close()


if __name__ == '__main__':
    observerd()


import datetime
import tabulate


class SimpleEventTextView:

    def __init__(self, event):
        self.event = event

    def __str__(self):
        return self.get_string()

    def get_string(self):
        string = ''.join([
            self._description_string(),
            self._location_string(),
            self._time_string(),
            self._creator_string()
        ])
        return string

    def _creator_string(self):
        string = (
            'Creator:\n'
            '   first name: {}\n'
            '   last name:  {}\n'
            '   id:         {}\n'
            '   affiliation:{}\n'
        ).format(
            self.event.creator.first_name,
            self.event.creator.last_name,
            self.event.creator.id,
            self.event.creator.affiliation
        )
        return string

    def _time_string(self):
        string = (
            'start time: {}\n'
        ).format(
            self.event.datetime.strftime('at %H:%M on %d th %m')
        )
        return string

    def _description_string(self):
        string = (
            'type:          {}\n'
            'title:         {}\n'
            'description:   {}\n'
        ).format(
            self.event.type,
            self.event.title,
            self.event.description
        )

        return string

    def _location_string(self):
        string = (
            'Location:\n'
            '   location:   {}\n'
            '   address:    {}\n'
        ).format(
            self.event.location,
            self.event.address
        )
        return string


class SimpleEventsTableView:
    """
    :ivar events: The list of events, that is supposed to be displayed by the view object
    :ivar event: The current event. Internally there is a loop through the list of events and methods of this class
        operate on this variable.
    :ivar max_length: The max length of characters allowed for one column of the table

    :cvar ORDER: A list of strings to be used as headers for the table.
        Reordering this list, will reorder the structure of the table
    :cvar DATETIME_FORMAT: The format string for the datetime string
    """
    ORDER = ['TITLE', 'TYPE', 'STARTING', 'CREATOR', 'LOCATION']
    DATETIME_FORMAT = '%d %B %Y, %H:%M:%S'

    def __init__(self, event_list, max_length=50):
        self.events = event_list
        self.event = None

        self.max_length = max_length

    def __str__(self):
        return self.get_string()

    def get_string(self):
        """
        The actual view string

        :return: string
        """
        table_list = self.table_list()
        string = tabulate.tabulate(table_list, headers=self.ORDER, tablefmt='fancy_grid')
        return string

    def table_list(self):
        """
        The list of rows for the table, which contains the row lists, each containing the items in the order specified
        by the class variable ORDER

        :return: [[string]]: A "matrix" structure of strings to display in the table
        """
        table_list = []
        for self.event in self.events:
            rows = self._event_rows()
            table_list.append(rows)
        return table_list

    def _event_rows(self):
        """
        The row list of the current event. The items are sorted by the ORDER variable given.

        :return: [string] The list of string items to be displayed in the table cells
        """
        # Getting the unordered dict, that assigns each cell item to one of the Header strings
        row_dict = self._event_row_dict()
        # Sorting the items into a list, that follows the same oder as the header
        row_list = []
        for key in self.ORDER:
            row_list.append(row_dict[key])
        return row_list

    def _event_row_dict(self):
        """
        The row dict for the current event. The items are sorted as values to the keys, which are the strings of the
        header items.

        :return: {string: string}
        """
        row_dict = {
            'TITLE': self._title_limited(),
            'TYPE': self.event.type,
            'STARTING': self._datetime_string(),
            'CREATOR': self._creator_full_name(),
            'LOCATION': self.event.location
        }
        return row_dict

    def _datetime_string(self):
        """
        The date & time of the current event, formatted by the format given as the class variable 'DATETIME_FORMAT'

        :return: string
        """
        return self.event.datetime.strftime(self.DATETIME_FORMAT)

    def _title_limited(self):
        """
        The title of the event, but stripped to the maximum number of character for the width of one column, if needed

        :return: string
        """
        if len(self.event.title) >= self.max_length:
            return '{}...'.format(self.event.title[:self.max_length])
        return self.event.title

    def _creator_full_name(self):
        """
        The full name of the creator of the event, separated by comma

        :return: string
        """
        return '{}, {}'.format(
            self.event.creator.last_name,
            self.event.creator.first_name
        )


from IndicoWp.wordpress import WordpressController
from IndicoWp.indico.controller import IndicoController
from IndicoWp.reference import ReferenceController


class Controller:

    def __init__(self):
        self.wordpress_controller = WordpressController()
        self.indico_controller = IndicoController()
        self.reference_controller = ReferenceController()

    def post_observed_events(self):
        # Getting all the observed events
        events = self.indico_controller.observed_events()
        # filter those already on the website
        references = self.reference_controller.select_all()
        reference_event_ids = list(map(lambda e: e.indico_id, references))

        events = list(filter(lambda e: e not in reference_event_ids, events))

        for event in events:
            # Actually posting the events to the website
            wordpress_id = self.wordpress_controller.post_event(event)
            # Adding a reference entry
            internal_event = self.reference_controller.event_from_indico_event(event)
            self.reference_controller.insert_reference(
                internal_event.id,
                event.id,
                wordpress_id
            )

    def close(self):
        self.reference_controller.close()


from sqlalchemy import Column, ForeignKey, Integer, String, Text, DATETIME
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import relationship, Session

from IndicoWp.database import MySQLDatabaseAccess, get_or_create
from IndicoWp.indico.event import IndicoEvent

Base = declarative_base()


class Creator(Base):
    __tablename__ = 'creator'

    id = Column(Integer, primary_key=True)
    first_name = Column(String(100))
    last_name = Column(String(100))
    affiliation = Column(String(300))


class Event(Base):
    __tablename__ = 'event'

    id = Column(Integer, primary_key=True)
    starting = Column(DATETIME)
    location = Column(String(200))
    address = Column(String(200))
    description = Column(Text)
    type = Column(String(100))
    title = Column(String(100))
    creator_id = Column(Integer, ForeignKey('creator.id'))

    creator = relationship(Creator)


class IndicoDatabaseController:

    def __init__(self):
        self.session = MySQLDatabaseAccess.get_session()  # type: Session

    def insert_multiple_events(self, event_list):
        for event in event_list:  # type: IndicoEvent
            self.insert_event(event)

    def insert_event(self, event):
        creator_model = get_or_create(
            self.session,
            Creator,
            id=event.creator.id,
            first_name=event.creator.first_name,
            last_name=event.creator.last_name,
            affiliation=event.creator.affiliation
        )

        event_model = get_or_create(
            self.session,
            Event,
            id=event.id,
            starting=event.datetime,
            location=event.location,
            address=event.address,
            description=event.description,
            type=event.type,
            title=event.title,
            creator_id=event.creator.id,
            creator=creator_model
        )


from IndicoWp.config import Config, PATH, PROJECT_PATH

import logging
import pathlib
import configparser
import json


class IndicoObservationController:

    def __init__(self):
        self.config = Config.get_instance()
        self.logger = logging.getLogger()

        self.path = pathlib.Path(PROJECT_PATH) / self.config['INDICO']['observe']
        self.observe = configparser.ConfigParser()
        self.observe.read(str(self.path))

        self.dict = {}
        self._load_observed_categories()

    def _load_observed_categories(self):
        for name in self.observe.keys():
            if name == 'DEFAULT':
                continue

            _dict = self.observe[name]
            url = _dict['url']
            key = _dict['key']
            category_list = json.loads(_dict['categories'])

            observation = IndicoObservation(
                name,
                category_list,
                url,
                key
            )

            self.dict[name] = observation

    def __getitem__(self, item):
        return self.dict[item]

    def keys(self):
        return self.dict.keys()

    def values(self):
        return self.dict.values()


class IndicoObservation:

    def __init__(self, name, category_list, url, key):
        self.name = name
        self.categories = category_list
        self.url = url
        self.key = key



from IndicoWp.indico.event import EventCreator, EventTime, EventLocation, EventDescription, IndicoEvent

import pytest


@pytest.fixture()
def event(request):
    # Creating a normal EventCreator
    event_creator = EventCreator(1, 'Max', 'Mustermann', 'KIT')
    # Creating default time
    event_time = EventTime('02:00:00', '2020-01-01')
    # Creating default description
    event_description = EventDescription('test', 'testing', 'seminar')
    # Creating default Location
    event_location = EventLocation('KIT', 'Karlsruhe')

    event = IndicoEvent(event_description, event_location, event_time, event_creator)
    return event


def test_event_object_basic(event):
    assert event.description == 'testing'
    assert event.title == 'test'
    assert event.type == 'seminar'
    assert event.location == 'KIT'
    assert event.address == 'Karlsruhe'
    assert event.creator.first_name == 'Max'
    assert event.creator.last_name == 'Mustermann'
    assert event.creator.affiliation == 'KIT'
    assert event.creator.id == 1
    assert event.datetime.hour == 2



from wordpress_xmlrpc import Client
from wordpress_xmlrpc import WordPressPost
from wordpress_xmlrpc.methods.posts import NewPost, EditPost, DeletePost, GetPost

from IndicoWp.config import Config

from IndicoWp.views.wordpress import IndicoEventWordpressPostView

import logging


class WordpressController:

    def __init__(self):
        self.logger = logging.getLogger('Wordpress')
        self.config = Config.get_instance()

        # Getting the url path to the xmlrpc.php file from the config file
        self.url = self.config['WORDPRESS']['url']
        # Getting the username and the password for the access to the wordpress api from the config file
        self.username = self.config['WORDPRESS']['username']
        self.password = self.config['WORDPRESS']['password']

        # Creating the client object from the login data
        self.client = Client(self.url, self.username, self.password)

    def post_event(self, indico_event):
        post_view = IndicoEventWordpressPostView(indico_event)

        post = WordPressPost()

        post.title = post_view.get_title()
        post.date = post_view.get_date()
        post.content = post_view.get_content()

        post.post_status = 'publish'
        post.comment_status = 'closed'

        post_id = self.client.call(NewPost(post))

        return post_id



import datetime


class EventLocation:
    """
    Parameter object, that represents the location of a Event and contains the string information about the location
    name and the address of the location.

    :ivar location: The string name of the location of the event
    :ivar address: The string of the address of the event
    """
    def __init__(self, location, address):
        self.location = location
        self.address = address


class EventCreator:
    """
    Parameter object, that represents the location of an Event and contains the information about the author of the
    Event on the indico platform, which includes the affiliation, id, first and last name.

    :ivar id: The string id of the author
    :ivar first_name: The string first name of the author
    :ivar last_name: The string last name of the author
    :ivar affiliation: The string info about the affiliation of the creator
    """
    def __init__(self, indico_id, first_name, last_name, affiliation):
        self.id = indico_id
        self.first_name = first_name
        self.last_name = last_name
        self.affiliation = affiliation


class EventDescription:
    """
    Parameter object, that represents the description of an event and contains the inf about the title, description and
    type of event.

    :ivar title: The title of the event
    :ivar description: The string description of the event
    :ivar type: The string definition of what type of event it is
    """
    def __init__(self, title, description, event_type):
        self.title = title
        self.description = description
        self.type = event_type


class EventTime:
    """
    Parameter object, that represents the time of an event, including the date and time it is supposed to take place

    :ivar time: The string time of the event. Has to have the format:
        "hour:minutes:seconds"
    :ivar date: The string date of the event. Has to have the format:
        "year-month-day"
    """
    DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'

    def __init__(self, time_string, date_string):
        self.time = time_string
        self.date = date_string

    @property
    def datetime(self):
        """
        Creates a new datetime object from the date and time string

        :return: datetime.datetime object for the time and date the event is supposed to take place
        """
        # Joining the string of the date and the time, so they match the format and can be converted to te datetime
        datetime_string = '{} {}'.format(self.date, self.time)

        datetime_object = datetime.datetime.strptime(datetime_string, self.DATETIME_FORMAT)
        return datetime_object


class EventMeta:

    def __init__(self, url, creation_time):
        self.url = url
        self.creation_time = creation_time


class IndicoEvent:
    """
    The data model for representing a Event of the indico website.
    It contains info about the description, type and title of the event, the location and address, the time it
    is supposed to take place and the event creator

    :ivar event_description: EventDescription object, containing the title, type and descritpion
    :ivar event_location: EventLocation containing the location name and the address
    :ivar event_time: EventTime containing starting time and date
    :ivar event_creator: EventCreator containing the creator name, id and affiliation
    """
    def __init__(self, indico_id, event_description, event_location, event_time, event_creator, event_meta):
        self.id = indico_id  # type: int
        self.event_description = event_description  # type: EventDescription
        self.event_location = event_location  # type: EventLocation
        self.event_time = event_time  # type: EventTime
        self.creator = event_creator  # type: EventCreator
        self.event_meta = event_meta  # type: EventMeta

    @property
    def url(self):
        return self.event_meta.url  # type: str

    @property
    def creation_time(self):
        return self.event_meta.creation_time  # type: EventTime

    @property
    def description(self):
        return self.event_description.description

    @property
    def type(self):
        return self.event_description.type

    @property
    def title(self):
        return self.event_description.title

    @property
    def location(self):
        return self.event_location.location

    @property
    def address(self):
        return self.event_location.address

    @property
    def datetime(self):
        return self.event_time.datetime

    @property
    def created(self):
        return self.creation_time.datetime

    def copy(self):
        event = IndicoEvent(
            self.id,
            self.event_description,
            self.event_location,
            self.event_time,
            self.creator,
            self.creation_time
        )
        return event



from IndicoWp.indico import IndicoEventRequestController, IndicoEventProcessor
from IndicoWp.indico.event import IndicoEvent

import pytest


@pytest.fixture()
def event_dict_basic(request):
    event_dict = {
        'creator': {
            'fullName': 'MUSTERMANN, Max',
            'id': '1',
            'affiliation': 'KIT'
        },
        'startDate': {
            'date': '2017-01-01',
            'time': '02:00:00'
        },
        'title': 'test',
        'description': 'testing',
        'type': 'seminar',
        'location': 'KIT',
        'address': 'Karlsruhe'
    }
    return event_dict


@pytest.fixture()
def event_processor(request):
    processor = IndicoEventProcessor()
    return processor


@pytest.fixture()
def event_requester(request):
    requester = IndicoEventRequestController()
    return requester


def test_indico_event_processing_basic(event_dict_basic, event_processor):
    event = event_processor.process(event_dict_basic)  # type: IndicoEvent

    assert event.creator.first_name == 'max'
    assert event.creator.last_name == 'mustermann'
    assert event.creator.id == 1

    assert event.datetime.hour == 2

    assert event.title == 'test'
    assert event.description == 'testing'
    assert event.type == 'seminar'

    assert event.location == 'KIT'
    assert event.address == 'Karlsruhe'


def test_indico_category_request_basic(event_requester):
    event_list = event_requester.get_category_events(384)
    assert isinstance(event_list, list)
    assert len(event_list) >= 1
    assert isinstance(event_list[0], IndicoEvent)



from IndicoWp.indico.observe import IndicoObservationController, IndicoObservation
from IndicoWp.indico.indico import IndicoEventRequestController
from IndicoWp.indico.persistency import IndicoDatabaseController


class IndicoController:

    def __init__(self):
        self.database_controller = IndicoDatabaseController()
        self.request_controller = IndicoEventRequestController()
        self.observation_controller = IndicoObservationController()

    def observed_events(self):
        event_list = []
        for observation in self.observation_controller.values():  # type: IndicoObservation
            self.request_controller.set(observation)
            for category in observation.categories:
                event_list += self.request_controller.get_category_events(category)
        return event_list

    def insert_events(self, event_list):
        self.database_controller.insert_multiple_events(event_list)



import logging
import configparser
import datetime
import pathlib
import os

# The path to the IndicoWp main folder, that contains all the modules
PATH = os.path.dirname(os.path.realpath(__file__))
VERSION = '0.1.0.5'
PROJECT_PATH = ''


class Config:
    """
    The static class, which acts as access point to the config parser object of this project.
    The config file read is the 'config.ini' in the main project folder.

    :cvar _instance: The static field containing the actual instance of the config parser
    :cvar _path: The pathlib.Path object pointing to the path of the config file
    """
    _instance = None
    _path = None

    @staticmethod
    def get_instance():
        """
        Returns the instance of the config file, that is saved in the singleton.

        :return: The configparser.ConfigParser object for this projects config file
        """
        if Config._instance is None:
            Config._create_instance()

        return Config._instance

    @staticmethod
    def _create_instance():
        """
        Actually creates the config parser upon first call of the get_instance method by reading the 'config.ini' file
        in the project folder.

        :return: void
        """
        Config._path = pathlib.Path(PROJECT_PATH) / 'config.ini'

        Config._instance = configparser.ConfigParser()
        Config._instance.read(str(Config._path))


class LoggingController:
    """
    Handles the log file management.
    Log files are being created for each day, with the date being part of the name of the log file.
    This class creates a new log file if there is none and init's the logging Logger for the file.
    Writes additional information into the log files at the beginning and end of a session.

    :cvar DATE_FORMAT: The datetime format string for dates only, used for the file names
    :cvar DATETIME_FORMAT: The datetime format string for date & time, used inside the log files
    :ivar logging_folder: The string name of the sub folder in which the logs are stored
    :ivar log_name: The name of the log file for the current day
    :ivar path: The pathlib.Path object to the folder in which the logs are located
    """
    DATE_FORMAT = '%Y_%m_%d'
    DATETIME_FORMAT = '%Y-%m-%d %H:%M:%S'

    BASE_NAME = 'indico'

    def __init__(self):

        # The name of the folder which will contain the log files
        self.config = Config.get_instance()
        self.logging_folder = self.config['LOGGING']['folder']
        self.log_name = self._create_log_name()

        # The path object pointing to the folder with the logs
        self.path = pathlib.Path(PROJECT_PATH) / self.logging_folder / self.log_name

    @property
    def path_string(self):
        """
        Returns the string path to the currently handled log file

        :return: string
        """
        return str(self.path)

    def init(self):
        """
        To be called before there is any attempt at writing log files.
        This creates a log file if there is none and also fixes the logging.logger on that file.

        :return: void
        """
        self._prepare_log_file()

    def close(self):
        """
        Writes a closing sentence into the log file.

        :return: void
        """
        self._post_process_log_file()

    def _prepare_log_file(self):
        """
        If there is no log file yet creates new one and appends a line, that states at which point a new session was
        started. Also inits the logging.BasicLogger on the newly created file.

        :return: void
        """
        datetime_string = self._current_datetime_string()

        line_string = '\n\nSTARTING NEW LOGGING SESSION @ "{}"\n\n'.format(datetime_string)

        file_mode = 'a'
        with self.path.open(mode=file_mode) as file:
            file.write(line_string)

        file_path = str(self.path)
        logging.basicConfig(
            level=logging.INFO,
            filename=file_path,
            filemode='a',
            format='%(asctime)s %(name)-40s %(levelname)-8s %(message)s'
        )

    def _post_process_log_file(self):
        """
        Writes a new line into the log file, that signals at which point the session ended

        :return: void
        """
        datetime_string = self._current_datetime_string()

        line_string = '\n\nCLOSING SESSION @ "{}"\n\n'.format(datetime_string)

        with self.path.open(mode='a') as file:
            file.write(line_string)

    def _create_log_name(self):
        """
        Creates the log name based on the BASE_NAME class variable and the current date.

        :return: The string name for the log file
        """
        # Creating the date string from the current date time object
        date_string = self._current_date_string()

        log_name = '{}_{}'.format(
            self.BASE_NAME,
            date_string
        )

        return log_name

    def _current_datetime_string(self):
        """
        Creates a datetime object for the current time of the method execution and formats the string based on the
        format, that is given as the DATETIME_FORMAT class variable

        :return: The string of the current date & time
        """
        datetime_object = datetime.datetime.now()
        datetime_string = datetime_object.strftime(self.DATETIME_FORMAT)

        return datetime_string

    def _current_date_string(self):
        """
        Creates a datetime object for the current moment of the method being executed and formats the string based on
        the format, that is given as the DATE_FORMAT class variable

        :return: A string for the current date
        """
        datetime_object = datetime.datetime.now()
        datetime_string = datetime_object.strftime(self.DATE_FORMAT)

        return datetime_string




from jinja2 import Template

from IndicoWp.config import Config, PATH
from IndicoWp.indico.event import IndicoEvent

import pathlib


class IndicoEventWordpressPostView:

    DATETIME_FORMAT = '%A, %d %B %Y at %M:%M'

    def __init__(self, event):
        self.event = event  # type: IndicoEvent

        # The path to the template file
        self.path = pathlib.Path(PATH) / 'templates' / 'post.html'
        self.template = None
        self.load_template()

    def load_template(self):
        with self.path.open(mode='r') as file:
            self.template = file.read()

    def get_title(self):
        title_encoded = self.event.title.encode('utf-8')
        return title_encoded

    def get_date(self):
        date_tuple = self.event.created.timetuple()
        return date_tuple

    def get_content(self):
        template = Template(self.template)
        content_string = template.render(
            location=self.event.location,
            address=self.event.address,
            description=self.event.description,
            starting=self._starting_date_string(),
            link=self._indico_link()
        )
        return content_string

    def _starting_date_string(self):
        return self.event.datetime.strftime(self.DATETIME_FORMAT)

    def _indico_link(self):
        return self.event.url



from setuptools import setup

setup(
    name='indico-wp',
    version='0.1.0.5',
    description='A tool for a wordpress server which will automatically post science events from indico website',
    url='https://github.com/the16thpythonist/IndicoWp',
    author='Jonas Teufel',
    author_email='jonseb1998@gmail.com',
    license='MIT',
    packages=['IndicoWp', 'IndicoWp/indico', 'IndicoWp/views', 'IndicoWp/tests', 'IndicoWp/templates'],
    include_package_data=False,
    install_requires=[
        'requests>=2.0',
        'mysqlclient>=1.2',
        'jinja2>=2.2',
        'SQLAlchemy>=1.2',
        'tabulate>=0.8',
        'python-wordpress-xmlrpc>=2.3'
    ],
    data_files=[('', ['/home/jonas/PycharmProjects/IndicoWp/IndicoWp/templates/post.html'])],
    python_requires='>=3, <4',
)



from IndicoWp.config import PATH, LoggingController, Config

import pathlib
import shutil

import pytest


@pytest.fixture(scope='session')
def log_folder(request):
    # Creating log folder
    log_folder = 'logging_tests'
    log_path = pathlib.Path(PATH) / log_folder

    # Now we need to replace the value that specifies the logging folder in the config file
    config = Config.get_instance()
    config['LOGGING']['folder'] = log_folder

    if not log_path.is_dir():
        log_path.mkdir()

    def delete_log_folder():
        log_path_string = str(log_path)
        shutil.rmtree(log_path_string)

    request.addfinalizer(delete_log_folder)


@pytest.mark.usefixtures('log_folder')
def test_log_file_creation():
    # Creating the logging controller
    logging_controller = LoggingController()

    logging_controller.init()
    log_path = logging_controller.path
    assert log_path.exists() and log_path.is_file()




from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, Session

from sqlalchemy.ext.declarative import declarative_base

from IndicoWp.config import Config


def get_or_create(session, model, **kwargs):
    instance = session.query(model).filter_by(**kwargs).first()
    if instance:
        return instance
    else:
        instance = model(**kwargs)
        session.add(instance)
        session.commit()
        return instance


class MySQLDatabaseAccess:

    _engine = None
    _session_maker = None

    @staticmethod
    def get_session():
        if MySQLDatabaseAccess._session_maker is None:
            MySQLDatabaseAccess._create_session_maker()

        return MySQLDatabaseAccess._session_maker()  # type: Session

    @staticmethod
    def create_database():
        Base = declarative_base()
        MySQLDatabaseAccess._create_engine()
        Base.metadata.create_all(MySQLDatabaseAccess._engine)

    @staticmethod
    def _create_session_maker():
        MySQLDatabaseAccess._create_engine()

        MySQLDatabaseAccess._session_maker = sessionmaker(bind=MySQLDatabaseAccess._engine)

    @staticmethod
    def _create_engine():
        config = Config.get_instance()
        username = config['MYSQL']['username']
        password = config['MYSQL']['password']
        host = config['MYSQL']['host']
        database = config['MYSQL']['database']

        engine_string = 'mysql+mysqldb://{}:{}@{}/{}'.format(
            username,
            password,
            host,
            database
        )

        MySQLDatabaseAccess._engine = create_engine(engine_string)
